import subprocess, time


class SocatHelper:
    SOCAT_CMD = "socat unix-listen:/socks/{service}.sock,fork,reuseaddr tcp-connect:{service}:{port}"

    def __init__(self, service, docker, path):
        self.service = service
        self.bridge = service
        self.container = f"{service}-socat"
        self.docker = docker
        self.path = path
        self.socks = path + "/socks"

        self.setup_socat()

    def bind(self, service, port):
        self.docker.exec_command(
            self.container,
            f"/bin/sh -c '{SocatHelper.SOCAT_CMD.format(service=service, port = port)}'",
            detach=True,
        )
        time.sleep(3)
        subprocess.call(["chmod", "666", f"{self.socks}/{service}.sock"])

    def unbind(self, service, port):
        pid = self.docker.exec_command(
            self.container,
            f"/bin/sh -c 'pgrep -f \"{SocatHelper.SOCAT_CMD.format(service=service, port = port)}\"'",
        ).output.decode("utf-8")[:-1]
        self.docker.exec_command(self.container, f"/bin/sh -c 'kill {pid}'")

    def bind_sockets(self, services):
        for service in services:
            self.bind(service.name, service.port)

    def setup_socat(self, force=False):
        print("Setting up socat, if needed...")

        docker = self.docker
        mounts = [docker.make_mount(self.socks, "/socks")]

        if not docker.network_exists(self.bridge):
            print(f"Creating {self.bridge} bridge network...")
            docker.create_network(self.bridge)

        if force or not docker.image_exists("socat:latest"):
            print("Building socat image...")

            with open(self.path + "/Dockerfile.socat", "w") as f:
                f.write(
                    "FROM alpine:edge\nARG VERSION=1.7.3.4-r1\nRUN apk --no-cache add socat=${VERSION}\n"
                )

            docker.build_image(
                path=self.path, dockerfile="Dockerfile.socat", tag="socat"
            )
            docker.wipe_container(self.container)

        if not docker.container_exists(self.container):
            print(f"Starting socat container at {self.container}...")
            con = docker.run_image(
                "socat",
                self.container,
                tty=True,
                detach=True,
                network=self.bridge,
                mounts=mounts,
            )

        print("Done! Socat setup complete.")
