from certbot._internal import cli, configuration, storage, main as cb_main
from certbot._internal.plugins import disco as plugins_disco


class Certbot:
    def __init__(self, args=[]):
        self.plugins = plugins_disco.PluginsRegistry.find_all()
        self.args = args

    def _config(self, args=[]):
        args = args + self.args
        args = cli.prepare_and_parse_args(self.plugins, args)
        return configuration.NamespaceConfig(args)

    def _cert_iter(self):
        for file in storage.renewal_conf_files(self._config()):
            yield storage.RenewableCert(file, self._config())

    def get_certs(self):
        return list(self._cert_iter())

    def cert_else_false(self, domain):
        domains = [domain, ".".join(["*"] + domain.split(".")[1:])]
        for cert in self._cert_iter():
            for d in domains:
                if d in cert.names():
                    return cert
        return False

    def attach_cert(self, cert, domain):
        self.run_bot([domain], ["install", "--cert-name", cert.live_dir.split("/")[-1]])

    def run_bot(self, domains, args=[]):
        args = list(args)
        for domain in domains:
            args.extend(["-d", domain])
        args.extend(self.args)
        cb_main.main(args)
