import docker
from docker.types import Mount


class Docker:
    def __init__(self):
        self.client = docker.from_env()
        self.api = docker.APIClient(base_url="unix://var/run/docker.sock")

    def network_exists(self, name):
        return name in [net.name for net in self.client.networks.list()]

    def get_network(self, name, low_level=False):
        for net in self.client.networks.list():
            if net.name == name:
                if low_level:
                    return self.api.inspect_network(net.id)
                return net

    def create_network(self, name):
        return self.client.networks.create(name, check_duplicate=True)

    def make_mount(self, host, internal):
        return Mount(internal, host, type="bind")

    def image_exists(self, name):
        for img in self.client.images.list():
            if name in img.tags:
                return True
        return False

    def pull_image(self, image, tag=None):
        self.client.images.pull(image, tag=tag)

    def build_image(self, **options):
        return self.client.images.build(**options)

    def prune_images(self):
        self.client.images.prune({"until": "240h"})

    def wipe_container(self, name):
        for con in self.client.containers.list(all=True):
            if name == con.name:
                if con.status == "running":
                    con.kill()
                con.remove()
                return name
        return "not found"

    def container_exists(self, name):
        return name in [con.name for con in self.client.containers.list(all=True)]

    def run_image(self, img, name, **options):
        return self.client.containers.run(img, name=name, **options)

    def exec_command(self, con, command, **options):
        if isinstance(con, str):
            con = self.client.containers.get(con)
        return con.exec_run(command, **options)

    def service_logs(self, con):
        if isinstance(con, str):
            con = self.client.containers.get(con)
        return con.logs(tail=100, timestamps=True).decode("utf-8")
