class Block:
    def __init__(self, name, *sections, **options):
        self.name = name
        self.sections = sections
        self.options = options

        if "ret" in self.options:
            self.options["return"] = self.options["ret"]
            del self.options["ret"]

    def repr_indent(self, indent=""):
        result = indent + self.name + " {\n"
        for block in self.sections:
            result += block.repr_indent(indent="    " + indent)
        for option in self.options:
            result += indent + "    " + option + " " + self.options[option] + ";\n"
        return result + indent + "}\n"

    def __repr__(self):
        return self.repr_indent(indent="")


class Server(Block):
    def __init__(self, *sections, **options):
        super().__init__("server", *sections, **options)


class Location(Block):
    def __init__(self, location, *sections, **options):
        super().__init__(f"location {location}", *sections, **options)


class Nginx:
    def __init__(self, default):
        self.default = default

    def gen_config_with_port(self, domain, port, logs_pre="/var/log/nginx/"):
        return self.gen_config(domain, f"http://127.0.0.1:{port}", logs_pre)

    def gen_config_with_sock(self, domain, sock, logs_pre="/var/log/nginx/"):
        return self.gen_config(domain, f"http://unix:{sock}", logs_pre)

    def gen_config(self, domain, proxy_pass, logs_pre):
        is_tld = len(domain.split(".")) == 2

        server_name = f"{domain}"
        if is_tld:
            server_name += f" www.{domain}"
        if domain == self.default:
            server_name += " default_server"

        http = Server(
            Location(
                "/",
                include="proxy_params",
                proxy_pass=proxy_pass,
            ),
            server_name=server_name,
            listen="80",
            access_log=f"{logs_pre}access.log",
            error_log=f"{logs_pre}error.log",
        )

        return str(http)
